# Generated by Django 3.2.13 on 2022-06-22 17:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0028_enterpriselearnerenrollment_offer_id'),
    ]

    operations = [
        migrations.CreateModel(
            name='EnterpriseOffer',
            fields=[
                ('offer_id', models.PositiveIntegerField(primary_key=True, serialize=False)),
                ('enterprise_customer_uuid', models.UUIDField()),
                ('enterprise_name', models.CharField(max_length=255, null=True)),
                ('sum_amount_learner_paid', models.FloatField(null=True)),
                ('sum_course_price', models.FloatField(null=True)),
                ('status', models.CharField(max_length=255, null=True)),
                ('offer_type', models.CharField(max_length=255, null=True)),
                ('date_created', models.DateTimeField(null=True)),
                ('emails_for_usage_alert', models.CharField(max_length=1023, null=True)),
                ('discount_type', models.CharField(max_length=64, null=True)),
                ('discount_value', models.FloatField(null=True)),
                ('max_discount', models.FloatField(null=True)),
                ('total_discount_ecommerce', models.FloatField(null=True)),
                ('amount_of_offer_spent', models.FloatField(null=True)),
                ('percent_of_offer_spent', models.FloatField(null=True)),
                ('remaining_balance', models.FloatField(null=True)),
            ],
            options={
                'verbose_name': 'Enterprise Offer',
                'verbose_name_plural': 'Enterprise Offers',
                'db_table': 'enterprise_offer_aggregates',
            },
        ),
    ]
